@@ -36,6 +36,7 @@ class interface(abstract_interface):
         self.output = {'X':0,'Y':0,'mag':0,'theta':0} #We define this also as class variable, to make it possible to see which data is produced by this interface without having to create an object
         self.continuous_read = False #When this is set to True, the data from device are acquired continuosly at the rate set by self.refresh_time
         self.refresh_time = 0.2 #default refresh rate, in seconds
+        self.sensitivity_auto = False #Boolean, true if software-controlled automatic sensitivity is enabled
         self.stored_data = {'X':[],'Y':[],'mag':[],'theta':[]}  # List used to store recorded data
         self.plot_window = None # QWidget object of the widget (i.e. floating window) that will contain the plot
         self.plot_object = None # PlotObject object of the plot where self.store_powers is plotted
@@ -197,6 +198,32 @@ class interface(abstract_interface):
             return False
         return True
 
+    def set_sensitivty_auto(self,status):
+        self.sensitivity_auto = status
+        if status==True:
+            self.read_sensitivity() #make sure to read the current value of sensitivity
+
+    def adjust_sensitivity_auto(self):
+        ''' Automatically adjust the sensitivity of the instrument based on the current value of magnitude.
+            It checks if the current magnitude value is above 20% and below 80% of the current sensitivity range.
+            If not, it adjust the sensitivity range to the smallest one such that the current value of magnitude is below 80% of it.
+        '''
+        max_bound = 0.8
+        min_bound = 0.2
+        current_mag = self.output['mag']
+        current_sensitivity_index = self.sensitivity
+        current_sensitivity_value = self.sensitivities[self.sensitivity]
+        if (current_sensitivity_value*min_bound) < current_mag < (current_sensitivity_value*max_bound):
+            return False #In this case we don't need to change the sensitivity
+        else:
+            self.logger.info(f"Need to adjust sensitivity.")
+            target_sensitivity = current_mag / max_bound
+            #Look for smallest sensitivity larger than target_sensitivity
+            index_new_sensitivity = self.sensitivities.index (
+                                                                min ([i for i in  self.sensitivities if i > target_sensitivity] )
+                                                                )
+            return self.set_sensitivity(index_new_sensitivity)
+
     def set_refresh_time(self, refresh_time):
         try: 
             refresh_time = float(refresh_time)
@@ -273,6 +300,10 @@ class interface(abstract_interface):
                     self.gui.edit_Output[k].setText(f"{self.output[k]:.2e}")
                 except:
                     self.gui.edit_Output[k].setText(f"nan")
+
+            if self.sensitivity_auto == True:
+                self.adjust_sensitivity_auto()
+
             QtCore.QTimer.singleShot(int(self.refresh_time*1e3), self.update)
            
         return
@@ -309,12 +340,19 @@ class gui(abstract_gui):
         self.combo_Sensitivities = Qt.QComboBox()
         self.combo_Sensitivities.activated.connect(self.click_combo_sensitivities)
 
+        self.box_SensitivityAuto = Qt.QCheckBox("Auto")
+        if self.interface.sensitivity_auto:
+            self.box_SensitivityAuto.setChecked(True)
+        self.box_SensitivityAuto.stateChanged.connect(self.click_box_SensitivityAuto)
+        self.box_SensitivityAuto.setToolTip('Enables a software-controlled adjustement of sensitivity.')
+
         hbox2.addWidget(self.button_ConnectDevice)
         hbox2.addWidget(self.button_SetAutoScale)
         hbox2.addWidget(self.label_TimeConstants)
         hbox2.addWidget(self.combo_TimeConstants,stretch=1)
         hbox2.addWidget(self.label_Sensitivities)
         hbox2.addWidget(self.combo_Sensitivities,stretch=1)
+        hbox2.addWidget(self.box_SensitivityAuto)
         #hbox2.addStretch(1)
 
         hbox3 = Qt.QHBoxLayout()
@@ -406,7 +444,8 @@ class gui(abstract_gui):
                                                self.label_Sensitivities,
                                                self.combo_Sensitivities,
                                                self.button_StartPauseReading,
-                                               self.button_StopReading]
+                                               self.button_StopReading,
+                                               self.box_SensitivityAuto]
 
         self.widgets_enabled_when_disconnected = [self.combo_Devices , 
                                                   self.button_RefreshDeviceList]
@@ -459,6 +498,14 @@ class gui(abstract_gui):
 
     def click_combo_sensitivities(self):
         self.interface.set_sensitivity(self.combo_Sensitivities.currentIndex())
+
+    def click_box_SensitivityAuto(self,state):
+        if state == QtCore.Qt.Checked:
+            status_bool = True
+        else:
+            status_bool = False
+        self.interface.set_sensitivty_auto(status_bool)
+        return
        
     def click_button_StartPauseReading(self): 
         if(self.interface.continuous_read == False):
